'''
Created on Nov 9, 2011

@author: Jakub Chmelar
'''

# import math, because we'll need some functions
import math

# in default state, debug messages are turned off
DEBUG = False

# angular delta for 'curve to polygon' conversion
ANGULAR_DELTA = math.pi/20

# --- classes ---

class Point():
    def __init__(self, x, y):
        self.x = x
        self.y = y
        
    def GetXY(self):
        return [self.x, self.y]

class Circle():
    def __init__(self, centre, radius):
        self.centre = centre
        self.radius = radius
        
class Intersection():
    def __init__(self, point, circle1, circle2):
        self.point = point
        self.circle1 = circle1
        self.circle2 = circle2

# --- methods ---

def DistanceAnglePoint(origin, distance, angle):
    x1, y1 = origin.GetXY()
    
    x2 = x1 + distance * math.cos(angle)
    y2 = y1 + distance * math.sin(angle)

    return Point(x2, y2)

def PointPointAngle(point1, point2):
    # get coordinates
    x1, y1 = point1.GetXY()
    x2, y2 = point2.GetXY()
    
    # get distance and deltas
    dx = x2 - x1
    dy = y2 - y1
    
    # get angle
    angle = math.atan2(dy, dx)
    
    return angle
    
def PointPointDistance(point1, point2):
    # get coordinates of point1
    x1, y1 = point1.GetXY()

    # get coordinates of point1
    x2, y2 = point2.GetXY()
    
    # calculate distance using Pythagoras
    d = math.sqrt(pow(x1-x2, 2) + pow(y1-y2, 2))
    
    # return distance
    return d    

def CircleCircleIntersection(circle1, circle2):
    
    # get centre and radius for circle1
    x1, y1 = circle1.centre.GetXY()
    r1 = circle1.radius

    if(DEBUG):
        print "x1 = %f, y1 = %f, r1 = %f" % (x1, y1, r1)

    # get centre and radius for circle2
    x2, y2 = circle2.centre.GetXY()
    r2 = circle2.radius

    if(DEBUG):
        print "x2 = %f, y2 = %f, r2 = %f" % (x2, y2, r2)

    # get centre-centre distance
    d = PointPointDistance(circle1.centre, circle2.centre)

    if(DEBUG):
        print "d = %f" % d

    # check for possible intersections
    # first condition: separate circles
    if(d > r1+r2):
        if(DEBUG):
            print "ERROR: circles can't intersect (separate circles)"
        return False
    
    # second condition: one circle within other circle
    elif(d < math.fabs(r1-r2)):
        if(DEBUG):
            print "ERROR: circles can't intersect (circle within circle)"
        return False
    
    else:
        if(DEBUG):
            print "OK: circles can intersect each other"

        # circle-circle intersection calculation
        # inspired by http://paulbourke.net/geometry/2circle/
        
        # a = (r1^2 - r2^2 + d^2)/(2*d)  
        a = (math.pow(r1, 2) - math.pow(r2, 2) + math.pow(d, 2))/(2*d)
        if(DEBUG):
            print "a = %f" % a
        
        # h^2 = r1^2 - a^2
        h_sq = math.pow(r1, 2) - math.pow(a, 2)
        if(DEBUG):
            print "h^2 = %f" % h_sq
        
        h = math.sqrt(h_sq)
        if(DEBUG):
            print "h = +-%f" % h
        
        # P3 = P1 + a ( P2 - P1 ) / d
        x3 = x1 + a*(x2-x1)/d
        y3 = y1 + a*(y2-y1)/d
        if(DEBUG):
            print "x3 = %f, y3 = %f" % (x3, y3)
        
        # x4 = x3 +- h ( y2 - y1 ) / d
        x4_1 = x3 + h*(y2-y1)/d
        x4_2 = x3 - h*(y2-y1)/d
        
        # y4 = y3 -+ h ( x2 - x1 ) / d 
        y4_1 = y3 - h*(x2-x1)/d
        y4_2 = y3 + h*(x2-x1)/d
        
        if(DEBUG):
            print "a) x4 = %f, y4 = %f" % (x4_1, y4_1)
            print "b) x4 = %f, y4 = %f" % (x4_2, y4_2)

        # create two intersection points
        int1 = Point(x4_1, y4_1)
        int2 = Point(x4_2, y4_2)
        
        # and return them
        return [int1, int2]
    
def NearestIntersection(point, path, intersections):
    nearest = None
    
    if(not (path or point)):
        return intersections[0]
    
    for i in intersections:
        if(i.circle1 == path or i.circle2 == path):
            if(nearest):
                if(PointPointDistance(point, i.point) < PointPointDistance(point, nearest.point)):
                    nearest = i
            else:
                nearest = i

    return nearest

def PolygonRawArea(points):
    # make local copy of points
    points = points[:]
    
    # check if first and last are same and fix it
    if(points[0] != points[len(points) - 1]):
        # copy first point to last
        points.append(points[0])

        if(DEBUG):
            print "Appending first point of polygon as last"   
    
    # init
    sum = 0
    
    # for all sides
    for i in range(len(points) - 1):
        # get this and next points
        p = points[i]
        p_1 = points[i+1] 
        
        # get coords of points
        x, y = p.GetXY()
        x_1, y_1 = p_1.GetXY()
        
        # get sum
        sum += (x*y_1 - x_1*y)
        
    # return sum divided by 2
    return sum/2.0

def PolygonArea(points):
    # get raw area -- signed
    area = PolygonRawArea(points)
    
    # fix sign
    if(area < 0):
        area = -area
        
    # return area, now always positive number
    return area

def PolygonCentroid(points):
    # make local copy of points
    points = points[:]
    
    # check if first and last are same and fix it
    if(points[0] != points[len(points) - 1]):
        # copy first point to last
        points.append(points[0])

        if(DEBUG):
            print "Appending first point of polygon as last"    
    
    # get polygon area (with sign)
    area = PolygonRawArea(points)
    
    # init
    sum_x = 0
    sum_y = 0
    
    # for all sides
    for i in range(len(points) - 1):
        # get this and next points
        p = points[i]
        p_1 = points[i+1] 
        
        # get coords of points
        x, y = p.GetXY()
        x_1, y_1 = p_1.GetXY()
    
        # calculate delta
        delta_x = (x + x_1)*(x*y_1 - x_1*y)
        delta_y = (y + y_1)*(x*y_1 - x_1*y) 
    
        if(DEBUG):
            print "delta_x = %.2f, delta_y = %.2f" % (delta_x, delta_y)

        # get sums
        sum_x += delta_x 
        sum_y += delta_y
        
        if(DEBUG):
            print "sum_x = %.2f, sum_y = %.2f" % (sum_x, sum_y)

    # get coordinates
    x = sum_x/(6.0*area)        
    y = sum_y/(6.0*area)
    
    # and finally return centroid point
    return Point(x, y) 

def GlobalIntersection(circles):
    # list for saving "good" intersections
    intersections = []

    # for all circle combinations
    for i in range(len(circles)):
        for j in range(i+1, len(circles)):
            # find intersections of these two circles
            result = CircleCircleIntersection(circles[i], circles[j])
        
            # if there are any intersections
            if(result):
                # get these as two points
                int1, int2 = result

                # set up error variables
                err1 = False
                err2 = False
                
                # check if intersections are inside rest of circles
                for k in range(len(circles)):
                    # do not check this for circles which made the intersection
                    if(k == i or k == j):
                        continue

                    # we can skip this check, if there already was error                    
                    if(err1 == False):
                        # if first intersection lies out of tested circle
                        if(PointPointDistance(int1, circles[k].centre) > circles[k].radius):
                            # save if as error
                            err1 = True

                    # we can skip this check, if there already was error
                    if(err2 == False):
                        # if second intersection lies out of tested circle
                        if(PointPointDistance(int2, circles[k].centre) > circles[k].radius):
                            # save if as error
                            err2 = True

                # now check error state for first intersection
                if(not err1):
                    # if it's valid, save it into list
                    intersection = Intersection(int1, circles[i], circles[j])
                    intersections.append(intersection)

                # now check error state for second intersection
                if(not err2):
                    # if it's valid, save it into list
                    intersection = Intersection(int2, circles[i], circles[j])
                    intersections.append(intersection)
    
    return intersections

def IntersectionsToPolygon(circles, intersections):
    # variable for saving last path used
    path = None
    
    # variable to store last point created
    p = None
    
    # list for points for polygon
    polygon = []
    
    # if we have any intersections
    if(intersections):
        # while we have some intersections left
        while(len(intersections)):
            # get nearest intersection by path and point
            i = NearestIntersection(p, path, intersections)
            
            # save intersection point into polygon
            polygon.append(i.point)
            
            # remove this intersection from list
            intersections.remove(i)
        
            # get circles creating intersection
            c1 = i.circle1
            c2 = i.circle2
    
            # if c1 was the path last time, swap them
            if(path and (path == c1)):
                c1, c2 = c2, c1
    
            # save c1 as path
            path = c1
            
            # find correct direction of path
            # get angle from c1.centre to i.point
            alfa = PointPointAngle(c1.centre, i.point)
            
            # angular delta
            delta = ANGULAR_DELTA
            
            # try one side
            test = DistanceAnglePoint(c1.centre, c1.radius, alfa + delta)
            
            # check test point if it is inside c2
            if(PointPointDistance(test, c2.centre) < c2.radius):
                # save positive direction
                direction = 1
            else:
                # save negative direction
                direction = -1
    
            # now we have direction, so we can travel along path
            # 1000 is limit value in case of some problems, instead of infinite loop
            for i in range(1000):
                # increase angle
                alfa += direction*delta
                
                # get new point
                p = DistanceAnglePoint(c1.centre, c1.radius, alfa)
                
                # error field for check
                error = False
                
                # check if point is inside other circles
                for c in circles:
                    # if it's same circle as path, do not check it
                    if(c == path):
                        continue
                    
                    # if it's outside
                    if(PointPointDistance(c.centre, p) > c.radius):
                        # set error
                        error = True
    
                        # stop it here as more iterations won't help
                        break
                    
                # if we are outside
                if(error):
                    # stop iteration
                    break
                
                # if we are inside
                else:
                    # finally save point for polygon
                    polygon.append(p)
                    
        # return polygon
        return polygon
                    
    # if we don't have any intersection
    # there is still possibility of circle within other circles
    else:
        # find circle with smallest radius
        smallest = circles[0]
        
        for c in circles:
            if(c.radius < smallest.radius):
                smallest = c
    
        # reset error state
        error = False
    
        # now check any point on it's outline
        p = DistanceAnglePoint(smallest.centre, smallest.radius, 0)
        
        # if it's inside all other circles
        for c in circles:
            # skip this check for smallest circle
            if(c == smallest):
                continue
            
            # check distance
            if(PointPointDistance(p, c.centre) > c.radius):
                # set error
                error = True
                
                # break for cycle, as more iterations are useless
                break
            
        # if there was no error
        if(error == False):
            # there must be circle within circle
            
            # let's transform it into polygon
            alfa = 0
            
            while(alfa < 2 * math.pi):
                # get new angle
                alfa += ANGULAR_DELTA
                
                # get point
                p = DistanceAnglePoint(smallest.centre, smallest.radius, alfa)
                
                # append point to polygon
                polygon.append(p)

            if(DEBUG):
                print "Circle within all other circles!"

            # return polygon we just created
            return polygon
        
        # if there was error
        else:
            # there are no intersections at all
            print "No intersections possible!"
            
            return False
    
