#!/usr/bin/env python

'''
Created on Dec 7, 2011

@author: Jakub Chmelar
'''

# import needed modules
import sys, json, random, math, SOI.cartesian

# enable debug?
DEBUG = False

# define speed of internet
# unit is km/ms
SPEED = 4/9.0 * 300

# set custom angular delta (optional) 
# SOI.cartesian.ANGULAR_DELTA = 0.1

# ---------------------------------------

# at first decode arguments as JSON strings
# reference data
reference_data = json.loads(sys.argv[1])

# target data
target_data = json.loads(sys.argv[2])

# ---------------------------------------

# decode target information

# prepare list of targets
targets = []

# process targets from JSON
for trgt in target_data:
    hostname = str(trgt['hostname'])
    latitude = float(trgt['latitude'])
    longitude = float(trgt['longitude'])
    
    # print debug info
    if(DEBUG):
        print "---"
        print "target:"
        print "  hostname\t= %s" % hostname
        print "  latitude\t= %.2f" % latitude
        print "  longitude\t= %.2f" % longitude
    
    # append to list
    targets.append([hostname, latitude, longitude, trgt])

# ---------------------------------------

# decode reference points information

# prepare list of reference points
references = []

# process reference points from JSON
for ref in reference_data:
    hostname = str(ref['hostname'])
    latitude = float(ref['latitude'])
    longitude = float(ref['longitude'])
    
    # print debug info
    if(DEBUG):
        print "---"
        print "reference:"
        print "  hostname\t= %s" % hostname
        print "  latitude\t= %.2f" % latitude
        print "  longitude\t= %.2f" % longitude
        
    # append to list
    references.append([hostname, latitude, longitude])

# ---------------------------------------

# now it's time to do geolocation with syntetic data

# prepare result list
results = []

# for all targets
for t in targets:
    # convert latitude/longitude to x/y cartesian   
    tx = t[1] * 111.330
    ty = t[2] * 64.152
    
    # print debug info
    if(DEBUG):
        print "---"
        print "target:"
        print "  hostname\t= %s" % t[0]
        print "  latitude->x\t= %.2f km" % tx
        print "  longitude->y\t= %.2f km" % ty
        print ""
    
    # create target point
    tp = SOI.cartesian.Point(tx, ty)
    
    # prepare circle list
    circles = []

    # ---------------------
    
    # for all reference points
    for r in references:
        # convert latitude/longitude to x/y cartesian   
        rx = r[1] * 111.330
        ry = r[2] * 64.152
        
        # print debug info
        if(DEBUG):
            print "reference:"
            print "  hostname\t= %s" % r[0]
            print "  latitude->x\t= %.2f km" % rx
            print "  longitude->y\t= %.2f km" % ry
            print ""
            
        # create reference point
        rp = SOI.cartesian.Point(rx, ry)
        
        # get real target <-> reference point distance
        real_distance = SOI.cartesian.PointPointDistance(tp, rp)
        
        # convert real distance to minimal latency using speed of internet
        real_latency = real_distance / SPEED
        
        # create higher random pseudolatency
        multiplier = 1.0 + random.random() * 0.5
        soi_latency = multiplier * real_latency
        
        # convert pseudolatency to pseudodistance
        soi_distance = soi_latency * SPEED
        
        # print debug info
        if(DEBUG):
            print "target <-> reference point:"
            print "  real distance\t= %.2f km" % real_distance
            print "  real latency\t= %.2f ms" % real_latency
            print ""
            print "  multiplier\t= %.2fx" % multiplier
            print "  SOI latency\t= %.2f ms" % soi_latency
            print "  SOI distance\t= %.2f km" % soi_distance
            print ""
            
        # now it's time to create circles
        c = SOI.cartesian.Circle(rp, soi_distance)
        
        # append it to list
        circles.append(c)

    # ---------------------
        
    # now we have complete circle list for this target
    
    # it's time to get intersections
    intersections = SOI.cartesian.GlobalIntersection(circles)
    
    # convert intersections to polygon
    polygon = SOI.cartesian.IntersectionsToPolygon(circles, intersections)
    
    # calculate polygon area
    area = SOI.cartesian.PolygonArea(polygon)
    
    # calculate polygon centroid
    centroid = SOI.cartesian.PolygonCentroid(polygon)

    # get centroid coordinates in x/y cartesian
    x, y = centroid.GetXY()

    # calculate deltas from real to estimated position
    dx = math.fabs(tx-x) 
    dy = math.fabs(ty-y)
    d = SOI.cartesian.PointPointDistance(tp, centroid)

    # print debug info
    if(DEBUG):
        print "---"
        print "results:"
        print "  polygon area\t= %.2f km^2" % area
        print "  polygon x\t= %.2f km" % x
        print "  polygon y\t= %.2f km" % y
        print ""
        print "  delta x\t= %.2f km" % dx
        print "  delta y\t= %.2f km" % dy
        print "  distance\t= %.2f km" % d

    
    # convert x/t cartesian to latitude/longitude
    latitude = x / 111.330
    longitude = y / 64.152
        
    # create output JSON data
    raw_data = t[3]

    # insert calculated data
    raw_data['latitude'] = "%.6f" % latitude
    raw_data['longitude'] = "%.6f" % longitude
    raw_data['image'] = "SOI.png"
    
    # append to result list
    results.append(raw_data)

# print debug info
if(DEBUG):
    print "---"
    print "results:"
    
# encode results to JSON and print it
print json.dumps(results)